package sqlindexwizard.application;

import org.apache.log4j.Logger;

import sqlindexwizard.clustering.WorkloadCompression;
import sqlindexwizard.indexadvisor.IndexAdvisor;
import sqlindexwizard.util.Log4jConfiguration;

/**
 * Main Class for the program.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class Main {
	
	private static final String[] LOGFILENAMES = {};
	
	/**
	 * Configures logging.
	 */
	static {
		Log4jConfiguration.configureLogging();
	}
	
	private static Logger logger = Log4jConfiguration.getLogger();
	
	/**
	 * Main-method of the program. Reads in Workload from files, performs workload
	 * compression and does index selection.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		logger.info("Starting application!");
		// starting workload compression using queries from logfiles
		if (args.length > 0 && args[0].equals("-nowc")) {
			WorkloadCompression wc = null;
			String[] args2 = new String[args.length-1];
			for (int i = 1; i < args.length; i++) {
				args2[i-1] = args[i];
			}
			if (args2.length > 0) {
				wc = new WorkloadCompression(args2);
			} else {
				if (LOGFILENAMES.length > 0) {
					wc = new WorkloadCompression(LOGFILENAMES);
				} else {
					System.out.println("ERROR: No input-file(s) given. Call SQLIndexWizard with input-file(s) as parameter!");
					System.exit(-1);
				}
			}
			// starting index advisor (for uncompressed and compressed wl for tests)
			IndexAdvisor idx1 = null;
			idx1 = new IndexAdvisor(wc.getWInitial());
			System.out.println("----------------------");
			System.out.println("     Index-Advisor");
			System.out.println("----------------------");
			idx1.goIndexAdvisor();
			System.out.println("\n\n" + idx1);
			logger.info("Finished application without workload compression!");
			System.exit(0);
		} else if (args.length > 0 && args[0].equals("-wconly")) {
			WorkloadCompression wc = null;
			String[] args2 = new String[args.length-1];
			for (int i = 1; i < args.length; i++) {
				args2[i-1] = args[i];
			}
			if (args2.length > 0) {
				wc = new WorkloadCompression(args2);
			} else {
				if (LOGFILENAMES.length > 0) {
					wc = new WorkloadCompression(LOGFILENAMES);
				} else {
					System.out.println("ERROR: No input-file(s) given. Call SQLIndexWizard with input-file(s) as parameter!");
					System.exit(-1);
				}
			}
			wc.goClustering();
			// printing output
			System.out.println("----------------------");
			System.out.println(" Workload-Compression ");
			System.out.println("----------------------");
			System.out.println();
			System.out.println(wc);
			logger.info("Finished application with WC only!");
			System.exit(0);
		} else if (args.length > 0 && args[0].equals("-testinitial")) {
			WorkloadCompression wc = null;
			String[] args2 = new String[args.length-1];
			for (int i = 1; i < args.length; i++) {
				args2[i-1] = args[i];
			}
			if (args2.length > 0) {
				wc = new WorkloadCompression(args2);
			} else {
				if (LOGFILENAMES.length > 0) {
					wc = new WorkloadCompression(LOGFILENAMES);
				} else {
					System.out.println("ERROR: No input-file(s) given. Call SQLIndexWizard with input-file(s) as parameter!");
					System.exit(-1);
				}
			}
			wc.goClustering();
			// printing output
			System.out.println("----------------------");
			System.out.println(" Workload-Compression ");
			System.out.println("----------------------");
			System.out.println();
			System.out.println(wc);
			IndexAdvisor idx1 = new IndexAdvisor(wc.getWInitial());
			IndexAdvisor idx2 = new IndexAdvisor(wc.getWCompressed());
			System.out.println("----------------------");
			System.out.println("     Index-Advisor");
			System.out.println("----------------------");
			idx1.goIndexAdvisor();
			idx2.goIndexAdvisor();
			System.out.println("\n\nRecommended Indexes WInitial:\n");
			System.out.println(idx1);
			System.out.println("\nRecommended Indexes WCompressed:\n");
			System.out.println(idx2);
		} else {
			WorkloadCompression wc = null;
			if (args.length > 0) {
				wc = new WorkloadCompression(args);
			} else {
				if (LOGFILENAMES.length > 0) {
					wc = new WorkloadCompression(LOGFILENAMES);
				} else {
					System.out.println("ERROR: No input-file(s) given. Call SQLIndexWizard with input-file(s) as parameter!");
					System.exit(-1);
				}
			}
			wc.goClustering();
			System.out.println("----------------------");
			System.out.println(" Workload-Compression ");
			System.out.println("----------------------");
			System.out.println();
			System.out.println(wc);
			IndexAdvisor idx1 = new IndexAdvisor(wc.getWCompressed());
			System.out.println("----------------------");
			System.out.println("     Index-Advisor");
			System.out.println("----------------------");
			idx1.goIndexAdvisor();
			System.out.println("\n\n" + idx1);
		}		
		logger.info("Finished application!");
	}

}
