package sqlindexwizard.clustering;

import java.util.Iterator;

import org.apache.log4j.Logger;

/**
 * Executes the Leader Algorithm. 
 * 
 * @author Benjamin Daeumlich
 *
 */
public class LeaderAlgo {

	private static Logger logger = Logger.getLogger(LeaderAlgo.class);

	/**
	 * Executes the leader algorithm.
	 * 
	 * @param QVList qvs to cluster
	 * @param leaderQVList inital Cluster
	 * @param boundValue bound values
	 * @return clusters
	 */
	public static ListOfQueryVectors start(ListOfQueryVectors QVList,
			ListOfQueryVectors leaderQVList, double boundValue) {
		logger.debug("Starting LeaderAlgo!");
		if (QVList == null) {
			return null;
		}
		ListOfQueryVectors leaderList;
		if (leaderQVList == null) {
			leaderList = new ListOfQueryVectors();
		} else {
			leaderList = leaderQVList;
		}
		Iterator<QueryVector> it = QVList.iterator();
		// handling first query vector
		if (leaderList.size() == 0 && it.hasNext()) {
			leaderList.add(it.next());
		}
		// handling further query vectors
		while (it.hasNext()) {
			QueryVector QV = it.next();
			// default values
			int clusterNumber = 0;
			double sim = 0;
			for (int i = 0; i < leaderList.size(); i++) {
				double simComp = leaderList.get(i).sim(QV);
				if (simComp > sim) {
					// new cluster membership
					sim = simComp;
					clusterNumber = i;
				}
			}
			if (sim >= boundValue) {
				// found cluster, increase number of elements in cluster
				leaderList.get(clusterNumber).incrementClusterSize(QV.getQ().getCount());
			} else {
				// new cluster for this query
				leaderList.add(QV);
			}
		}
		logger.debug("Finished LeaderAlgo - LeaderSize = " + leaderList.size() + "!");
		return leaderList;
	}
}
