package sqlindexwizard.clustering;

import java.util.ArrayList;
import java.util.List;

import sqlindexwizard.filter.JoinElement;
import sqlindexwizard.filter.SelectElement;
import sqlindexwizard.workload.DeleteQuery;
import sqlindexwizard.workload.InsertQuery;
import sqlindexwizard.workload.SelectQuery;
import sqlindexwizard.workload.UpdateQuery;

/**
 * Representing a partition. In a partition are similar query-vectors.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class Partition {

	private ListOfQueryVectors qvList = new ListOfQueryVectors();
	private ArrayList<String> tables = new ArrayList<String>();
	private ArrayList<String> columns = new ArrayList<String>();
	private List<JoinElement> joinList = new ArrayList<JoinElement>();
	private ArrayList<SelectElement> groupByList = new ArrayList<SelectElement>();
	private ArrayList<SelectElement> orderByList = new ArrayList<SelectElement>();
	private ArrayList<SelectElement> selectList = new ArrayList<SelectElement>();
	private String having = "";
	
	/**
	 * Constructor adds qv to the list and sets tables.
	 * 
	 * @param qv
	 */
	public Partition(QueryVector qv) {
		this.tables = qv.getQ().getTables();
		if (qv.getQ() instanceof SelectQuery) {
			this.joinList = ((SelectQuery)qv.getQ()).getJoinList();
			this.groupByList = ((SelectQuery)qv.getQ()).getGroupBy();
			this.orderByList = ((SelectQuery)qv.getQ()).getOrderBy();
			this.having = ((SelectQuery)qv.getQ()).getHaving();
			ArrayList<SelectElement> selectListTemp = ((SelectQuery)qv.getQ()).getSelect();
			for (int i = 0; i < selectListTemp.size(); i++) {
				if (selectListTemp.get(i).getType() > 0) {
					selectList.add(selectListTemp.get(i));
				}
			}
		} else if (qv.getQ() instanceof UpdateQuery) {
			this.columns = ((UpdateQuery)qv.getQ()).getColumns();
		} else if (qv.getQ() instanceof InsertQuery) {
			this.columns = ((UpdateQuery)qv.getQ()).getColumns();
		} else if (qv.getQ() instanceof DeleteQuery) {
			//nothing to do
		}
		qvList.add(qv);
	}
	
	/**
	 * Adds qv and returns true if qv belongs to this partition, else returns false.
	 * 
	 * @param qv qv to add
	 * @return true, if qv belongs to this partition
	 */
	public boolean add(QueryVector qv) {
		ArrayList<String> tablesQv = qv.getQ().getTables();
		if (qv.getQ() instanceof SelectQuery) {
			List<JoinElement> joinListQv = ((SelectQuery)qv.getQ()).getJoinList();
			ArrayList<SelectElement> groupByListQv = ((SelectQuery)qv.getQ()).getGroupBy();
			ArrayList<SelectElement> orderByListQv = ((SelectQuery)qv.getQ()).getOrderBy();
			ArrayList<SelectElement> selectListQv = new ArrayList<SelectElement>();
			ArrayList<SelectElement> selectListQvTemp = ((SelectQuery)qv.getQ()).getSelect();
			String havingQv = ((SelectQuery)qv.getQ()).getHaving();
			for (int i = 0; i < selectListQvTemp.size(); i++) {
				if (selectListQvTemp.get(i).getType() > 0) {
					selectListQv.add(selectListQvTemp.get(i));
				}
			}
			if ((this.tables.size() == tablesQv.size()) && this.tables.containsAll(tablesQv)
					&& (this.joinList.size() == joinListQv.size()) && this.joinList.containsAll(joinListQv)
					&& (this.orderByList.size() == orderByListQv.size()) && this.orderByList.equals(orderByListQv)
					&& (this.groupByList.size() == groupByListQv.size()) && this.groupByList.equals(groupByListQv)
					&& (this.selectList.size() == selectListQv.size()) && this.selectList.containsAll(selectListQv)
					&& (this.having.equals(havingQv)) ) {
				qvList.add(qv);
			} else {
				return false;
			}
		} else if (qv.getQ() instanceof UpdateQuery) {
			ArrayList<String> columnsQv = ((UpdateQuery)qv.getQ()).getColumns();
			if ((this.tables.size() == tablesQv.size()) && this.tables.containsAll(tablesQv)
					&& (this.columns.size() == columnsQv.size()) && this.columns.containsAll(columnsQv) ) {
				qvList.add(qv);
			} else {
				return false;
			}
		} else if (qv.getQ() instanceof InsertQuery) {
			ArrayList<String> columnsQv = ((UpdateQuery)qv.getQ()).getColumns();
			if ((this.tables.size() == tablesQv.size()) && this.tables.containsAll(tablesQv)
					&& (this.columns.size() == columnsQv.size()) && this.columns.containsAll(columnsQv) ) {
				qvList.add(qv);
			} else {
				return false;
			}
		} else if (qv.getQ() instanceof DeleteQuery) {
			if ((this.tables.size() == tablesQv.size()) && this.tables.containsAll(tablesQv)) {
				qvList.add(qv);
			} else {
				return false;
			}
		}
		return false;
	}
	
	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(Object o) {
		if (o instanceof Partition) {
			return equals((Partition) o);
		} else {
			return false;
		}
	}

	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(Partition part) {
		if (part.getQvList().get(0).getQ() instanceof SelectQuery) {
			if ((this.tables.size() == part.tables.size()) && this.tables.containsAll(part.tables)
					&& (this.joinList.size() == part.joinList.size()) && this.joinList.containsAll(part.joinList)
					&& (this.groupByList.size() == part.groupByList.size()) && this.groupByList.equals(part.groupByList)
					&& (this.orderByList.size() == part.orderByList.size()) && this.orderByList.equals(part.orderByList)
					&& (this.selectList.size() == part.selectList.size()) && this.selectList.containsAll(part.selectList)
					&& (this.having.equals(part.having))) {
				return true;
			} else {
				return false;
			}
		} else if (part.getQvList().get(0).getQ() instanceof UpdateQuery) {
			if ((this.tables.size() == part.tables.size()) && this.tables.containsAll(part.tables)
					&& (this.columns.size() == part.columns.size()) && this.columns.containsAll(part.columns) ) {
				return true;
			} else {
				return false;
			}
		} else if (part.getQvList().get(0).getQ() instanceof InsertQuery) {
			if ((this.tables.size() == part.tables.size()) && this.tables.containsAll(part.tables)
					&& (this.columns.size() == part.columns.size()) && this.columns.containsAll(part.columns) ) {
				return true;
			} else {
				return false;
			}
		} else if (part.getQvList().get(0).getQ() instanceof DeleteQuery) {
			if ((this.tables.size() == part.tables.size()) && this.tables.containsAll(part.tables)) {
				return true;
			} else {
				return false;
			}
		}
		return false;
	}

	/**
	 * Getting the ListOfQueryVectors.
	 * 
	 * @return the qvList
	 */
	public ListOfQueryVectors getQvList() {
		return qvList;
	}
	
	/**
	 * String representation of Partition
	 * 
	 * @return string representation
	 */
	public String toString() {
		String result = "[Tables: " + tables.toString() + " - Size: " + this.qvList.size() + " - Joins: " + this.joinList + " - Select: " + this.selectList + " - GroupBy: " + this.groupByList + "]";
		return result;
	}
}
