package sqlindexwizard.clustering;

/**
 * Represents an item in the query vector.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class QueryVectorItem implements Cloneable{

	private String tableName;
	private String columnName;
	
	/**
	 * Constructor sets tableName and columnName
	 * 
	 * @param tableName table name
	 * @param columnName column name
	 */
	public QueryVectorItem(String tableName, String columnName) {
		this.tableName = tableName;
		this.columnName = columnName;
	}
	
	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(Object o) {
		if (o instanceof QueryVectorItem) {
			return equals((QueryVectorItem) o);
		} else {
			return false;
		}
	}
	
	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(QueryVectorItem it) {
		return (this.tableName.equals(it.tableName) && this.columnName.equals(it.columnName));
	}
	
	/**
	 * Generating hashCode for QueryVectorItem.
	 * 
	 * @return hashCode
	 */
	public int hashCode() {
		return this.tableName.hashCode() + this.columnName.hashCode();
	}
	
	/**
	 * Clones the QueryVectorItem.
	 * 
	 * @return clone
	 */
	public QueryVectorItem clone() {
		return new QueryVectorItem(this.tableName, this.columnName);
	}
	
	/**
	 * String representation of the QueryVectorItem.
	 * 
	 * @return string representation
	 */
	public String toString() {
		return this.tableName + "." + this.columnName;
	}
	
}
