package sqlindexwizard.clustering;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.log4j.Logger;

import sqlindexwizard.dbconnection.ConnectionDB2;
import sqlindexwizard.dbconnection.TableInfoDB2;

/**
 * Provides information about a table.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class TableElement {

	private String tableName;
	private int tCard;
	private ArrayList<String> columns = new ArrayList<String>();
	private HashMap<String, String> colTypes = new HashMap<String, String>();
	private HashMap<String, Integer> colCard = new HashMap<String, Integer>();
	private HashMap<String, String> high2Keys = new HashMap<String, String>();
	private HashMap<String, String> low2Keys = new HashMap<String, String>();
	private HashMap<String, Integer> nrFreqVals = new HashMap<String, Integer>();
	private HashMap<String, Integer> nrQuantiles = new HashMap<String, Integer>();
	
	private static Logger logger = Logger.getLogger(ConnectionDB2.class);
	
	/**
	 * Constructor generates a new TableElement for the table tableName.
	 * 
	 * @param tableName
	 */
	public TableElement(String tableName) {
		logger.debug("Generating new TableElement for Table " + tableName + "!");
		TableInfoDB2 tabInfo = new TableInfoDB2();
		this.tableName = tableName.toUpperCase();
		tabInfo.generateStatistics(tableName);
		tCard = tabInfo.getTCard(tableName);
		colTypes = tabInfo.getColTypes();
		columns.addAll(colTypes.keySet());
		colCard = tabInfo.getColCard();
		high2Keys = tabInfo.getHigh2Keys();
		low2Keys = tabInfo.getLow2Keys();
		nrFreqVals = tabInfo.getNrFreqVals();
		nrQuantiles = tabInfo.getNrQuantiles();
		tabInfo.finalize();
	}
	
	/**
	 * Getting coltype for column.
	 * 
	 * @param column column
	 * @return type
	 */
	public String getColType(String column) {
		return colTypes.get(column);
	}
	
	/**
	 * Getting colcard for column.
	 * 
	 * @param column column
	 * @return colcard
	 */
	public int getColCard(String column) {
		return colCard.get(column);
	}
	
	/**
	 * Getting high2key for column.
	 * 
	 * @param column column
	 * @return high2key
	 */
	public String getHigh2Key(String column) {
		return high2Keys.get(column);
	}
	
	/**
	 * Getting low2key for column.
	 * 
	 * @param column column
	 * @return low2key
	 */
	public String getLow2Key(String column) {
		return low2Keys.get(column);
	}
	
	/**
	 * Getting nrfreqvals for column.
	 * 
	 * @param column column
	 * @return nrfreqvals
	 */
	public int getNrFreqVals(String column) {
		return nrFreqVals.get(column);
	}
	
	/**
	 * Getting nrquantiles for column.
	 * 
	 * @param column column
	 * @return nrquantiles
	 */
	public int getNrQuantiles(String column) {
		return nrQuantiles.get(column);
	}
	
	/**
	 * Getting tCard for table.
	 * 
	 * @return tCard
	 */
	public int getTCard() {
		return tCard;
	}
	
	/**
	 * Getting columns for table.
	 * 
	 * @return columns
	 */
	public ArrayList<String> getColumns() {
		return columns;
	}
	
	/**
	 * A string representation for a TableElement.
	 * 
	 * @return string representation.
	 */
	public String toString() {
		String result = "";
		result += "Tablename:\t" + tableName + "\n";
		result += "TCard:\t" + tCard + "\n";
		result += "Columns:\t" + columns + "\n";
		result += "ColTypes:\t" + colTypes + "\n";
		result += "ColCards:\t" + colCard + "\n";
		result += "Low2Keys:\t" + low2Keys + "\n";
		result += "High2Keys:\t" + high2Keys + "\n";
		result += "NrFreqVals:\t" + nrFreqVals + "\n";
		result += "NrQuantiles:\t" + nrQuantiles + "\n";
		return result;
	}
}
