package sqlindexwizard.dbconnection;

import java.sql.*;

import org.apache.log4j.Logger;

import com.ibm.db2.jcc.DB2DataSource;

import sqlindexwizard.util.Config;

/**
 * Connects to the database.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class ConnectionDB2 {
	
	private Connection co = null;
	private Statement qStmt = null;

	private static Logger logger = Logger.getLogger(ConnectionDB2.class);
	
	/**
	 * Constructor creates database connection.
	 * 
	 */
	public ConnectionDB2() {
		createDBConnection();
	}
	
	/**
	 * Cloeses database connection.
	 * 
	 */
	public void finalize() {
		closeDBConnection();
	}
	
	/**
	 * Creating database connection, getting connection properties from config-file.
	 * 
	 */
	private void createDBConnection() {
		try {
			DB2DataSource datasource = new DB2DataSource();
			datasource.setServerName(Config.getPropertyString("db2.host"));
			datasource.setUser(Config.getPropertyString("db2.user"));
			datasource.setPassword(Config.getPropertyString("db2.password"));
			datasource.setDriverType(4);
			datasource.setPortNumber(Integer.valueOf(Config.getPropertyString("db2.port")));
			datasource.setDatabaseName(Config.getPropertyString("db2.database"));
			co = datasource.getConnection();
		} catch (Exception e){
			logger.error("getConnection failed(): " + e.toString());
			System.out.println("Cannot connect to Database, please check connection properties in sqlindexwizard.properties!");
			System.exit(-1);
		}
		logger.debug("DB connection established!");
	}
	
	/**
	 * Executes a query and returns the ResultSet.
	 * 
	 * @param query QueryString
	 * @return ResultSet
	 */
	public ResultSet executeQuery(String query) {
		logger.debug("Executing query " + query + ".");
		ResultSet rs = null;
		try {
			qStmt = co.createStatement();
			rs = qStmt.executeQuery(query);
			//stmt.close();
		} catch (SQLException e) {
			logger.error("Error in executeQuery(" + query + "): " + e.toString());
		} catch (Exception e) {
			logger.error("Error in executeQuery(" + query + "): " + e.toString());
		}
		return rs;
	}
	
	/**
	 * Executes a query.
	 * 
	 * @param query QueryString
	 * @return ErrorNr
	 */
	public int executeQueryNoResult(String query) {
		logger.debug("Executing query " + query + ".");
		try {
			Statement stmt = co.createStatement();
			stmt.execute(query);
			stmt.close();
		} catch (SQLException e) {
			if (e.toString().contains("ERRORCODE=-4228")) {
				logger.debug("Normal error in executeQueryNoResult(" + query + "): " + e.toString());
				return 0;
			} else if (e.toString().contains("SQLCODE=-104")) {
				logger.error("Parse Error in executeQueryNoResult(" + query + "): " + e.toString());
				return 1;
			} else {
				logger.error("Unknown Error in executeQueryNoResult(" + query + "): " + e.toString());
				return 1;
			}
		} catch (Exception e) {
			logger.debug("Unknown error in executeQueryNoResult(" + query + "): " + e.toString());
			return 1;
		}
		return 0;
	}
	
	/**
	 * Executes an update query.
	 * 
	 * @param query QueryString
	 */
	public void updateQuery(String query) {
		logger.debug("Executing query " + query + ".");
		try {
			Statement stmt = co.createStatement();
			stmt.executeUpdate(query);
			stmt.close();
		} catch (SQLException e) {
			logger.error("Error in executeQuery(" + query + "): " + e.toString());
		} catch (Exception e) {
			logger.error("Error in executeQuery(" + query + "): " + e.toString());
		}
	}
	
	/**
	 * Closing statement after resultset is not needed anymore.
	 * 
	 */
	public void closeStatement() {
		try {
			if (qStmt != null) {
				qStmt.close();
			}
		} catch (SQLException e) {
			logger.error("Error closing statement: " + e.toString());
		} catch (Exception e) {
			logger.error("Error closing statement: " + e.toString());
		}
	}
	
	/**
	 * Closing database connection.
	 * 
	 */
	private void closeDBConnection() {
		try {
			co.close();
		} catch (SQLException e) {
			logger.error("Error closing connection: " + e.toString());
		} catch (Exception e) {
			logger.error("Error closing connection: " + e.toString());
		}
		logger.debug("DB connection closed successfully!");
	}
	
}
