package sqlindexwizard.dbconnection;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Date;

import org.apache.log4j.Logger;

import sqlindexwizard.util.Config;

/**
 * This class runs the runstats command if it is necessary.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class RunstatsDB2 {

	private Date last;
	private boolean execute = false;
	private String cmd = "";
	
	private static Logger logger = Logger.getLogger(RunstatsDB2.class);
	
	/**
	 * Constructor, sets execute variable.
	 * 
	 */
	public RunstatsDB2() {
		setExecute();
	}
	
	/**
	 * Checks whether runstats execution is necessary.
	 * 
	 */
	public void setExecute() {
		try {
			String value = Config.getPropertyString("db2.lastRunstats");
			long runstatsPeriod = Long.parseLong(Config.getPropertyString("general.runstatsPeriod"));
			if (value.equals("-1")) {
				execute = true;
			} else {
				last = Date.valueOf(value);
				long lastLong = last.getTime();
				long todayLong = System.currentTimeMillis();
				if ((todayLong - lastLong) > (runstatsPeriod * 24 * 60 * 60 * 1000)) {
					execute = true;
				}
			}
		} catch (Exception e) {
			logger.error("Error in setExecute(): " + e.getMessage());
			execute = true;
		}
	}
	
	/**
	 * Getting execute.
	 * 
	 * @return execute
	 */
	public boolean getExecute() {
		return this.execute;
	}
	
	/**
	 * Adds runstats command for a given table to the cmd string.
	 * 
	 * @param table tablename
	 */
	public void addRunstatsCmd(String table) {
		String query = "RUNSTATS ON TABLE " + Config.getPropertyString("db2.schema") + "." + table + " ON ALL COLUMNS WITH DISTRIBUTION AND INDEXES ALL ALLOW WRITE ACCESS;\n";
		cmd += query;
	}
	
	/**
	 * Executes runstats.
	 * 
	 */
	public void run() {
		try {
			logger.info("Executing runstats!");
			File outputFile = new File("RunstatsCmd.db2");
			FileWriter out = new FileWriter(outputFile);
			out.write("CONNECT TO " + Config.getPropertyString("db2.database") + ";\n");
			out.write(cmd);
			out.write("CONNECT RESET;\n");
			out.close();
			Process p = Runtime.getRuntime().exec("db2cmd /c /w /i db2 -vtf RunstatsCmd.db2");
			
			// open streams for the process's input and error
			BufferedReader stdInput = new BufferedReader(new
		                                      InputStreamReader(p.getInputStream()));
			BufferedReader stdError = new BufferedReader(new
		                                      InputStreamReader(p.getErrorStream()));
			String s;
			// read the output from the command and set the output variable with
			// the value
			while ((s = stdInput.readLine()) != null) {
				logger.debug(s);
			}
			// read any errors from the attempted command and set the error
			// variable with the value
			while ((s = stdError.readLine()) != null) {
				logger.error("Error in executing db2 runstats: "+ s);
			}

			p.destroy();
			outputFile.deleteOnExit();
			logger.info("Finished executing runstats!");
		} catch (IOException e) {
			logger.error("IOException in run(): " + e.toString());
		} catch (Exception e) {
			logger.error("Exception in run(): " + e.toString());
		}
	}

}
