package sqlindexwizard.dbconnection;

import java.sql.*;
import java.util.HashMap;

import org.apache.log4j.Logger;

/**
 * Generates needed information for a table.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class TableInfoDB2 extends ConnectionDB2{
	
	private HashMap<String, String> colTypes = new HashMap<String, String>();
	private HashMap<String, Integer> colCard = new HashMap<String, Integer>();
	private HashMap<String, String> high2Keys = new HashMap<String, String>();
	private HashMap<String, String> low2Keys = new HashMap<String, String>();
	private HashMap<String, Integer> nrFreqVals = new HashMap<String, Integer>();
	private HashMap<String, Integer> nrQuantiles = new HashMap<String, Integer>();
	
	private static Logger logger = Logger.getLogger(TableInfoDB2.class);

	/**
	 * Constructor creates connection.
	 */
	public TableInfoDB2() {
		super();
	}
	
	/**
	 * Sets local variables, getting information from SYSIBM.SYSCOLUMNS.
	 * 
	 * @param tableName tablename
	 */
	public void generateStatistics(String tableName) {
		logger.debug("Generating table statistics for " + tableName);
		try {
			String query = "SELECT NAME,COLTYPE,COLCARD,NQUANTILES,NMOSTFREQ,LOW2KEY,HIGH2KEY FROM SYSIBM.SYSCOLUMNS WHERE TBNAME = '" + tableName.toUpperCase() + "'";
			ResultSet rs = executeQuery(query);
			while (rs.next()) {
				colTypes.put(rs.getString("NAME"),rs.getString("COLTYPE"));
				colCard.put(rs.getString("NAME"),Integer.valueOf(rs.getString("COLCARD")));
				low2Keys.put(rs.getString("NAME"),rs.getString("LOW2KEY"));
				high2Keys.put(rs.getString("NAME"),rs.getString("HIGH2KEY"));
				nrFreqVals.put(rs.getString("NAME"),Integer.valueOf(rs.getString("NMOSTFREQ")));
				nrQuantiles.put(rs.getString("NAME"),Integer.valueOf(rs.getString("NQUANTILES")));
			}
			rs.close();
			closeStatement();
		} catch (SQLException e) {
			logger.error("Error in generateStatistics(" + tableName + "): " + e.toString());
		} catch (Exception e) {
			logger.error("Error in generateStatistics(" + tableName + "): " + e.toString());
		}
	}
	
	/**
	 * Getting tCard from SYSIBM.SYSTABLES.
	 * 
	 * @param tableName tableName
	 * @return tCard
	 */
	public int getTCard(String tableName) {
		logger.debug("Getting tCard for table " + tableName);
		int count = 0;
		try {
			String query = "SELECT CARD FROM SYSIBM.SYSTABLES WHERE NAME = '" + tableName.toUpperCase() + "'";
			ResultSet rs = executeQuery(query);
			while (rs.next()) {
				count = Integer.valueOf(rs.getString("CARD"));
			}
			rs.close();
			closeStatement();
		} catch (SQLException e) {
			logger.error("Error in getNrOfTuples(" + tableName + "): " + e.toString());
		} catch (Exception e) {
			logger.error("Error in getNrOfTuples(" + tableName + "): " + e.toString());
		}
		return count;
	}

	/**
	 * Getting ColTypes.
	 * 
	 * @return colTypes
	 */
	public HashMap<String, String> getColTypes() {
		return colTypes;
	}

	/**
	 * Getting ColCard.
	 * 
	 * @return colCard
	 */
	public HashMap<String, Integer> getColCard() {
		return colCard;
	}

	/**
	 * Getting High2Keys.
	 * 
	 * @return High2Keys
	 */
	public HashMap<String, String> getHigh2Keys() {
		return high2Keys;
	}

	/**
	 * Getting Low2Keys.
	 * 
	 * @return Low2Keys
	 */
	public HashMap<String, String> getLow2Keys() {
		return low2Keys;
	}

	/**
	 * Getting nrFreqVals.
	 * 
	 * @return nrFreqVals
	 */
	public HashMap<String, Integer> getNrFreqVals() {
		return nrFreqVals;
	}

	/**
	 * Getting nrQuantiles.
	 * 
	 * @return nrQuantiles
	 */
	public HashMap<String, Integer> getNrQuantiles() {
		return nrQuantiles;
	}
	
}
