package sqlindexwizard.filter;

import java.util.ArrayList;
import java.util.List;

/**
 * AndWhereElement: WhereElement AND WhereElement
 * 
 * @author Benjamin Daeumlich
 *
 */
public class AndWhereElement extends WhereElement {

	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public AndWhereElement() {
		super();
	}

	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public AndWhereElement(ListOfWhereElements elements) {
		super(elements);
	}
	
	/**
	 * WhereElement is already in CNF. Call recursively for elements and remove useless
	 * parentheses.
	 * 
	 * @return WhereElement in CNF
	 */
	public WhereElement toCNF() {
		AndWhereElement cnf = new AndWhereElement();
		for (WhereElement where : elements) {
			cnf.add(where.toCNF());
		}
		if (cnf.size() == 0) return cnf;
		return cnf.removeParThisLevel();
	}
	
	/**
	 * Computes DNF of WhereElement. Call recursively for elements, multiplies and remove useless
	 * parentheses.
	 * 
	 * Part3: a and (b or c) = (a and b) or (a and c)
	 * 
	 * @return WhereElement in DNF
	 */
	public WhereElement toDNF() {
		WhereElement dnf = new AndWhereElement();
		for (WhereElement where: elements) {
			dnf.add(where.toDNF());
		}
		if (dnf.size() == 0) {
			return dnf;
		}
		dnf = dnf.removeParThisLevel();
		if (dnf instanceof AndWhereElement) {
			dnf = ((AndWhereElement)dnf).multiply();
		}
		return dnf;
	}
	
	/**
	 * Multiplies WhereElement. 
	 * 
	 * Part3: a and (b or c) = (a and b) or (a and c)
	 * 
	 * @return multiplied WhereElement
	 */
	public WhereElement multiply() {
		List<ListOfWhereElements> helpOR = new ArrayList<ListOfWhereElements>();
		ListOfWhereElements helpAND = new ListOfWhereElements();
		for (int i = 0; i < this.size(); i++) {
			if (this.get(i) instanceof OrWhereElement) {
				helpOR.add(this.get(i).elements);
			} else {
				helpAND.add(this.get(i));
			}
		}
		if (helpOR.size() == 0) {
			return new AndWhereElement(helpAND);
		}
		int newsize = 1;
		for (int i = 0; i < helpOR.size(); i++) {
			newsize *= helpOR.get(i).size();
		}
		OrWhereElement result = new OrWhereElement();
		int[] itemArray = new int[helpOR.size()];
		for (int i = 0; i < itemArray.length; i++) {
			itemArray[i] = 0;
		}
		for (int i = 0; i < newsize; i++) {
			boolean calculatedNextTuple = false;
			WhereElement help = new AndWhereElement();
			help.addAll(helpAND);
			for (int j = 0; j < helpOR.size(); j++) {
				help.add(helpOR.get(j).get(itemArray[j]));
				if (!calculatedNextTuple) {
					// Increment the value of the itemArray
					if (itemArray[j] >= helpOR.get(j).size() - 1) {
						// All Filters in this list added (cannot increment value)
						itemArray[j] = 0;
					} else {
						itemArray[j] = itemArray[j] + 1;
						calculatedNextTuple = true;
					}
				}
			}
			result.add(help.removeParThisLevel());
		}	
		return result;
	}
	
	/**
	 * Simplifies this WhereElement, not implemented yet.
	 * 
	 * @return this.clone()
	 */
	public WhereElement simplify() {
		// TODO Auto-generated method stub
		return this.clone();
	}
	
	/**
	 * Removes useless parentheses.
	 * 
	 * a and (b and c) = a and b and c
	 * 
	 * @return WhereElement without useless parentheses
	 */
	public WhereElement removeParThisLevel() {
		AndWhereElement result = new AndWhereElement();
		for (int i=0; i<this.size(); i++) {
			if (this.get(i) instanceof AndWhereElement) {
				result.addAll(this.get(i).elements);
			} else {
				result.add(this.get(i));
			}
		}
		return result;
	}
	
	/**
	 * String representation of this WhereElement.
	 * 
	 * @return string representation
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append('(');
		for (WhereElement where : elements) {
			buffer.append(where.toString());
			buffer.append(" AND ");
		}
		return buffer.substring(0, buffer.length() - 5) + ")";
	}

}
