package sqlindexwizard.filter;

import java.util.ArrayList;
import java.util.Vector;

import Zql.*;

/**
 * Represents an ItemWhereElement: column operator values.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class ItemWhereElement extends WhereElement {

	/**
	 * Type Integer.
	 */
	public static final byte TYPE_INTEGER = 1;
	/**
	 * Type Double.
	 */
	public static final byte TYPE_DOUBLE = 2;
	/**
	 * Type String.
	 */
	public static final byte TYPE_STRING = 3;
	/**
	 * Type Date.
	 */
	public static final byte TYPE_DATE = 4;
	/**
	 * Type Time.
	 */
	public static final byte TYPE_TIME = 5;
	/**
	 * Type Timestamp.
	 */
	public static final byte TYPE_TIMESTAMP = 6;
	/**
	 * Type Unknown.
	 */
	public static final byte TYPE_UNKNOWN = 7;
	
	private String operator = "";
	private String column = "";
	private String attributeString = "";
	private ArrayList<String> values = new ArrayList<String>();
	private ArrayList<String> possibleTables = new ArrayList<String>();
	private String table = "";
	private byte type = 0;
	private boolean isNot = false;
	private int count = 1;
	private boolean isJoin = false;
	private JoinElement joinElement = null;

	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public ItemWhereElement() {
		super();
	}

	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public ItemWhereElement(ListOfWhereElements elements) {
		super(elements);
	}

	/**
	 * Constructor sets local variables.
	 * 
	 * @param operator operator
	 * @param operands operands
	 */
	public ItemWhereElement(String operator, Vector<ZExp> operands) {
		this.operator = operator;
		this.attributeString = ((ZConstant)operands.get(0)).toString();
		processAttributeString(attributeString);
		for (int i=1; i < operands.size(); i++) {
			values.add(((ZConstant)operands.get(i)).toString());
			if (((ZConstant)operands.get(i)).getType() == ZConstant.COLUMNNAME) {
				isJoin = true;
				joinElement = new JoinElement(attributeString, values.get(0), operator);
			}
		}
	}
	
	/**
	 * Constructor sets local variables.
	 * 
	 * @param operator operator
	 * @param operands operands
	 */
	public ItemWhereElement(String operator, String[] operands) {
		this.operator = operator;
		this.attributeString = operands[0];
		processAttributeString(attributeString);
		for (int i=1; i < operands.length; i++) {
			values.add(operands[i]);
		}
	}
	
	/**
	 * Constructor sets local variables.
	 * 
	 * @param operator operator
	 * @param attributeString attributeString
	 * @param values values
	 */
	public ItemWhereElement(String operator, String attributeString, String[] values) {
		this.operator = operator;
		this.attributeString = attributeString;
		processAttributeString(attributeString);
		for (int i=0; i < values.length; i++) {
			this.values.add(values[i]);
		}
	}
	
	/**
	 * Constructor sets local variables.
	 * 
	 * @param Operator operator
	 * @param attributeString attributeString
	 * @param values values
	 */
	public ItemWhereElement(String Operator, String attributeString, ArrayList<String> values) {
		this.operator = Operator;
		this.attributeString = attributeString;
		processAttributeString(attributeString);
		this.values = values;
	}
	
	/**
	 * Tries to extract tableName and columnName from the attributeString.
	 * 
	 * @param attributeString 
	 */
	public void processAttributeString(String attributeString) {
		ZAliasedName att = new ZAliasedName(attributeString, ZAliasedName.FORM_COLUMN);
		this.column = att.getColumn().toUpperCase();
		if (att.getTable() != null) {
			table = att.getTable().toUpperCase();
		}
	}
	
	/**
	 * Generates CNF of this element, nothing to do.
	 * 
	 * @return this.clone();
	 */
	@Override
	public WhereElement toCNF() {
		return this.clone();
	}

	/**
	 * Generates DNF of this element, nothing to do.
	 * 
	 * @return this.clone();
	 */
	@Override
	public WhereElement toDNF() {
		return this.clone();
	}
	
	/**
	 * Simplifies this element, nothing to do.
	 * 
	 * @return this.clone();
	 */
	@Override
	public WhereElement simplify() {
		return this.clone();
	}
	
	/**
	 * Removes parentheses this level, nothing to do.
	 * 
	 * @return this.clone();
	 */
	@Override
	public WhereElement removeParThisLevel() {
		return this.clone();
	}
	
	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(Object o) {
		if (o instanceof ItemWhereElement) {
			return equals((ItemWhereElement) o);
		} else {
			return false;
		}
	}

	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(ItemWhereElement it) {
		if (!isJoin) {
			return (this.column.equals(it.column) && this.operator.equals(it.operator) && this.values.containsAll(it.values) && this.table.equals(it.table) && this.isNot == it.isNot && this.isJoin == it.isJoin);
		} else {
			return joinElement.equals(it.joinElement);
		}
	}
	
	/**
	 * Tests for equality of two Objects for Sim-Calculation.
	 * 
	 * @return true if they are equal
	 */
	public boolean equalsSim(ItemWhereElement it) {
		if (!isJoin) {
			return (this.column.equals(it.column) && this.operator.equals(it.operator) && this.table.equals(it.table) && this.isNot == it.isNot && this.isJoin == it.isJoin);
		} else {
			return joinElement.equals(it.joinElement);
		}
	}

	/**
	 * Generating hashCode for ItemWhereElement.
	 * 
	 * @return hashCode
	 */
	public int hashCode(){
		return this.column.hashCode() + this.operator.hashCode() + this.values.hashCode() + this.table.hashCode() + String.valueOf(this.isNot()).hashCode();
	}
	
	/**
	 * Clones the itemwhereelement
	 * 
	 * @return clone
	 */
	public ItemWhereElement clone() {
		ItemWhereElement clone = new ItemWhereElement(operator, attributeString, values);
		clone.setNot(this.isNot());
		clone.setJoin(this.isJoin());
		clone.setTable(this.table);
		clone.setJoinElement(this.joinElement);
		clone.setCount(this.count);
		return clone;
	}
	

	/**
	 * Returns true if the table of this Item is known.
	 * 
	 * @return true if table is set
	 */
	public boolean hasTable() {
		return (!table.equals(""));
	}

	/**
	 * Getting table.
	 * 
	 * @return table
	 */
	public String getTable() {
		return table.toUpperCase();
	}

	/**
	 * Setting table.
	 * 
	 * @param table new value
	 */
	public void setTable(String table) {
		this.table = table.toUpperCase();
	}

	/**
	 * Getting operator.
	 * 
	 * @return operator
	 */
	public String getOperator() {
		return operator;
	}

	/**
	 * Getting column.
	 * 
	 * @return column
	 */
	public String getColumn() {
		return column.toUpperCase();
	}

	/**
	 * Sets the type-variable.
	 * 
	 * @param typeString type as String
	 */
	public void setType(String typeString) {
		typeString = typeString.trim();
		if (typeString.equals("SMALLINT") || typeString.equals("INTEGER") || typeString.equals("BIGINT")) {
			this.type = TYPE_INTEGER;
		} else if (typeString.equals("REAL") || typeString.equals("DOUBLE") || typeString.equals("DECIMAL")) {
			this.type = TYPE_DOUBLE;
		} else if (typeString.equals("VARCHAR") || typeString.equals("CHAR")) {
			this.type = TYPE_STRING;
		} else if (typeString.equals("DATE")) {
			this.type = TYPE_DATE;
		} else if (typeString.equals("TIME")) {
			this.type = TYPE_TIME;
		} else if (typeString.equals("TIMESTAMP")) {
			this.type = TYPE_TIMESTAMP;
		} else {
			this.type = TYPE_UNKNOWN;
		}
	}

	/**
	 * Setting the type-variable to type.
	 * 
	 * @param type new type
	 */
	public void setType(byte type) {
		this.type = type;
	}
	
	/**
	 * Getting type.
	 * 
	 * @return type
	 */
	public byte getType() {
		return type;
	}

	/**
	 * Getting values.
	 * 
	 * @return values
	 */
	public ArrayList<String> getValues() {
		return values;
	}	
	
	/**
	 * Getting possible Tables.
	 * 
	 * @return possibleTables
	 */
	public ArrayList<String> getPossibleTables() {
		return possibleTables;
	}

	/**
	 * Setting possible Tables.
	 * 
	 * @param possibleTables new value
	 */
	public void setPossibleTables(ArrayList<String> possibleTables) {
		this.possibleTables = possibleTables;
	}
	
	/**
	 * Getting isNot.
	 * 
	 * @return the isNot
	 */
	public boolean isNot() {
		return isNot;
	}

	/**
	 * Setting isNot.
	 * 
	 * @param isNot the isNot to set
	 */
	public void setNot(boolean isNot) {
		this.isNot = isNot;
	}

	/** 
	 * Getting isJoin.
	 * 
	 * @return isJoin
	 */
	public boolean isJoin() {
		return isJoin;
	}

	/**
	 * Setting isJoin.
	 * 
	 * @param isJoin value
	 */
	public void setJoin(boolean isJoin) {
		this.isJoin = isJoin;
	}

	/**
	 * Getting count.
	 * 
	 * @return the count
	 */
	public int getCount() {
		return count;
	}

	/**
	 * Setting count.
	 * 
	 * @param count the count to set
	 */
	public void setCount(int count) {
		this.count = count;
	}
	
	/**
	 * Increasing count by 1.
	 * 
	 */
	public void increaseCount() {
		this.count++;
	}

	/**
	 * Getting joinElement.
	 * 
	 * @return the joinElement
	 */
	public JoinElement getJoinElement() {
		return joinElement;
	}

	/**
	 * Setting joinElement.
	 * 
	 * @param joinElement the joinElement to set
	 */
	public void setJoinElement(JoinElement joinElement) {
		this.joinElement = joinElement;
	}

	/**
	 * String representation of this WhereElement.
	 * 
	 * @return string representation
	 */
	public String toString() {
		String result = "(";
		if (isJoin()) {
			return "(" + joinElement.toString() + ")";
		}
		if (hasTable()) {
			result += table + ".";
		}
		result += column + " " + operator + " ";
		if (operator.equals("BETWEEN")) {
			result += values.get(0) + " AND " + values.get(1);
		} else if (operator.equals("IN")) {
			result += "(";
			for (int i = 0; i < values.size(); i++) {
				result += values.get(i) + ", ";
			}
			result = result.substring(0, result.length()-2);
			result += ")";
		} else {
			if (values.size() > 0) {
				result += values.get(0);
			} else {
				result = result.substring(0, result.length()-1);
			}
		}
		result += ")";
		return result;
	}
	
}
