package sqlindexwizard.filter;

import java.util.ArrayList;
import Zql.ZAliasedName;

/**
 * Representation of a join.
 * 
 * @author Benjamin Daeumlich
 *
 */

public class JoinElement {

	private String operator = "";
	private String columnLeft = "";
	private String columnRight = "";
	private String attributeStringLeft = "";
	private String attributeStringRight = "";
	private ArrayList<String> possibleTables = new ArrayList<String>();
	private String tableLeft = "";
	private String tableRight = "";
	
	/**
	 * Constructor processed attribute-strings and sets tables and columns.
	 * 
	 * @param attributeStringLeft
	 * @param attributeStringRight
	 * @param operator
	 */
	public JoinElement(String attributeStringLeft, String attributeStringRight, String operator) {
		this.operator = operator;
		this.attributeStringLeft = attributeStringLeft;
		this.attributeStringRight = attributeStringRight;
		processAttributeStringLeft(attributeStringLeft);
		processAttributeStringRight(attributeStringRight);
	}
	
	/**
	 * Tries to extract tableName and columnName from the attributeString.
	 * 
	 * @param attributeString 
	 */
	public void processAttributeStringLeft(String attributeString) {
		ZAliasedName att = new ZAliasedName(attributeString, ZAliasedName.FORM_COLUMN);
		this.columnLeft = att.getColumn().toUpperCase();
		if (att.getTable() != null) {
			tableLeft = att.getTable().toUpperCase();
		}
	}
	
	/**
	 * Tries to extract tableName and columnName from the attributeString.
	 * 
	 * @param attributeString 
	 */
	public void processAttributeStringRight(String attributeString) {
		ZAliasedName att = new ZAliasedName(attributeString, ZAliasedName.FORM_COLUMN);
		this.columnRight = att.getColumn().toUpperCase();
		if (att.getTable() != null) {
			tableRight = att.getTable().toUpperCase();
		}
	}
	
	/**
	 * Returns true if the left table of this Item is known.
	 * 
	 * @return true if table is set
	 */
	public boolean hasTableLeft() {
		return (!tableLeft.equals(""));
	}
	
	/**
	 * Returns true if the right table of this Item is known.
	 * 
	 * @return true if table is set
	 */
	public boolean hasTableRight() {
		return (!tableRight.equals(""));
	}
	
	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(Object o) {
		if (o instanceof JoinElement) {
			return equals((JoinElement) o);
		} else {
			return false;
		}
	}

	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(JoinElement it) {
		if (it == this) {
			return true;
		} else if (it == null) {
			return false;
		} else {
			return ((this.columnRight.equals(it.columnRight) && this.operator.equals(it.operator) && this.columnLeft.equals(it.columnLeft) && this.tableLeft.equals(it.tableLeft) && this.tableRight.equals(it.tableRight)) ||
					(this.columnRight.equals(it.columnLeft) && this.operator.equals(it.operator) && this.columnLeft.equals(it.columnRight) && this.tableLeft.equals(it.tableRight) && this.tableRight.equals(it.tableLeft)));
		}
	}	
	
	/**
	 * Generating hashCode for JoinElement.
	 * 
	 * @return hashCode
	 */
	public int hashCode(){
		return this.columnLeft.hashCode() + this.columnRight.hashCode() + this.operator.hashCode() + this.tableLeft.hashCode() + this.tableRight.hashCode();
	}
	
	/**
	 * Clones the JoinElement
	 * 
	 * @return clone
	 */
	public JoinElement clone() {
		JoinElement clone = new JoinElement(this.attributeStringLeft, this.attributeStringRight, operator);
		return clone;
	}

	/**
	 * Getting the columnLeft.
	 * 
	 * @return the columnLeft
	 */
	public String getColumnLeft() {
		return columnLeft;
	}

	/**
	 * Setting the columnLeft.
	 * 
	 * @param columnLeft the columnLeft to set
	 */
	public void setColumnLeft(String columnLeft) {
		this.columnLeft = columnLeft;
	}

	/**
	 * Getting the possibleTables.
	 * 
	 * @return the possibleTables
	 */
	public ArrayList<String> getPossibleTables() {
		return possibleTables;
	}

	/**
	 * Setting the possibleTables.
	 * 
	 * @param possibleTables the possibleTables to set
	 */
	public void setPossibleTables(ArrayList<String> possibleTables) {
		this.possibleTables = possibleTables;
	}

	/**
	 * Getting the tableLeft.
	 * 
	 * @return the tableLeft
	 */
	public String getTableLeft() {
		return tableLeft;
	}

	/**
	 * Setting the tableLeft.
	 * 
	 * @param tableLeft the tableLeft to set
	 */
	public void setTableLeft(String tableLeft) {
		this.tableLeft = tableLeft.toUpperCase();
	}

	/**
	 * Getting the tableRight.
	 * 
	 * @return the tableRight
	 */
	public String getTableRight() {
		return tableRight;
	}

	/**
	 * Setting the tableRight.
	 * 
	 * @param tableRight the tableRight to set
	 */
	public void setTableRight(String tableRight) {
		this.tableRight = tableRight.toUpperCase();
	}

	/**
	 * Getting the operator.
	 * 
	 * @return the operator
	 */
	public String getOperator() {
		return operator;
	}

	/**
	 * Getting columnRight.
	 * 
	 * @return the columnRight
	 */
	public String getColumnRight() {
		return columnRight;
	}

	/**
	 * Getting the attributeStringLeft.
	 * 
	 * @return the attributeStringLeft
	 */
	public String getAttributeStringLeft() {
		return attributeStringLeft;
	}

	/**
	 * Getting the attributeStringRight.
	 * 
	 * @return the attributeStringRight
	 */
	public String getAttributeStringRight() {
		return attributeStringRight;
	}
	
	/**
	 * String representation of this JoinElement.
	 * 
	 * @return string representation
	 */
	public String toString() {
		String result = "";
		if (hasTableLeft()) {
			result += tableLeft + ".";
		}
		result += columnLeft + " " + this.operator + " ";
		if (hasTableRight()) {
			result += tableRight + ".";
		}
		result += columnRight;		
		return result;
	}

}
