package sqlindexwizard.filter;

/**
 * NotWhereElement NOT(WhereElement).
 * 
 * @author Benjamin Daeumlich
 *
 */
public class NotWhereElement extends WhereElement {

	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public NotWhereElement() {
		super();
	}

	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public NotWhereElement(WhereElement element) {
		super();
		add(element);
	}

	/**
	 * Computes CNF of WhereElement. Perform part2, remove useless
	 * parentheses and multiply.
	 * 
	 * Part 2:
	 * 
	 * 		NOT(a or b) = NOT(a) and NOT(b)
	 * 		NOT(a and b) = NOT(a) or NOT(b)
	 *      NOT(NOT(a)) = a
	 * 
	 * @return WhereElement in CNF
	 */
	public WhereElement toCNF() {
		WhereElement cnf;
		if (this.get(0) instanceof AndWhereElement) {
			cnf = new OrWhereElement();
			for (int i = 0; i < this.get(0).size(); i++) {
				cnf.add(new NotWhereElement(this.get(0).get(i)).toCNF());
			}
		} else if (this.get(0) instanceof OrWhereElement) {
			cnf = new AndWhereElement();
			for (int i = 0; i < this.get(0).size(); i++) {
				cnf.add(new NotWhereElement(this.get(0).get(i)).toCNF());
			}
		} else if (this.get(0) instanceof NotWhereElement) {
			return this.get(0).get(0).toCNF();
		} else if (this.get(0) instanceof TautologyWhereElement) {
			return new ContradictionWhereElement();
		} else if (this.get(0) instanceof ContradictionWhereElement) {
			return new TautologyWhereElement();
		} else if (this.get(0) instanceof ItemWhereElement) {
			return this;
		} else {
			return this;
		}
		cnf = cnf.removeParThisLevel();
		if (cnf instanceof OrWhereElement) {
			cnf = ((OrWhereElement)cnf).multiply();
		}
		return cnf;
	}
	
	/**
	 * Computes DNF of WhereElement. Perform part2, remove useless
	 * parentheses and multiply.
	 * 
	 * Part 2:
	 * 
	 * 		NOT(a or b) = NOT(a) and NOT(b)
	 * 		NOT(a and b) = NOT(a) or NOT(b)
	 *      NOT(NOT(a)) = a
	 * 
	 * @return WhereElement in DNF
	 */
	public WhereElement toDNF() {
		WhereElement dnf;
		if (this.get(0) instanceof AndWhereElement) {
			dnf = new OrWhereElement();
			for (int i = 0; i < this.get(0).size(); i++) {
				dnf.add(new NotWhereElement(this.get(0).get(i)).toDNF());
			}
		} else if (this.get(0) instanceof OrWhereElement) {
			dnf = new AndWhereElement();
			for (int i = 0; i < this.get(0).size(); i++) {
				dnf.add(new NotWhereElement(this.get(0).get(i)).toDNF());
			}
		} else if (this.get(0) instanceof NotWhereElement) {
			return this.get(0).get(0).toDNF();
		} else if (this.get(0) instanceof TautologyWhereElement) {
			return new ContradictionWhereElement();
		} else if (this.get(0) instanceof ContradictionWhereElement) {
			return new TautologyWhereElement();
		} else if (this.get(0) instanceof ItemWhereElement) {
			return this;
		} else {
			return this;
		}
		dnf = dnf.removeParThisLevel();
		if (dnf instanceof AndWhereElement) {
			dnf = ((AndWhereElement)dnf).multiply();
		}
		return dnf;
	}
	
	/**
	 * Simplifies this WhereElement, not implemented yet.
	 * 
	 * @return this.clone()
	 */
	public WhereElement simplify() {
		// TODO Auto-generated method stub
		return this.clone();
	}
	
	/**
	 * Removes useless parentheses, nothing to do.
	 * 
	 * @return this.clone()
	 */
	public WhereElement removeParThisLevel() {
		return this.clone();
	}
	
	/**
	 * String representation of this WhereElement.
	 * 
	 * @return string representation
	 */
	public String toString() {
		return "NOT (" + super.toString() + ")";
	}

}
