package sqlindexwizard.filter;

import java.util.ArrayList;
import java.util.List;

/**
 * OrWhereElement: WhereElement OR WhereElement
 * 
 * @author Benjamin Daeumlich
 *
 */
public class OrWhereElement extends WhereElement {

	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public OrWhereElement() {
		super();
	}

	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public OrWhereElement(ListOfWhereElements elements) {
		super(elements);
	}

	/**
	 * Computes CNF of WhereElement. Call recursively for elements, multiplies and remove useless
	 * parentheses.
	 * 
	 * Part3: a or (b and c) = (a or b) and (a or c)
	 * 
	 * @return WhereElement in CNF
	 */
	public WhereElement toCNF() {
		WhereElement cnf = new OrWhereElement();
		for (WhereElement where: elements) {
			cnf.add(where.toCNF());
		}
		if (cnf.size() == 0) {
			return cnf;
		}
		cnf = cnf.removeParThisLevel();
		if (cnf instanceof OrWhereElement) {
			cnf = ((OrWhereElement)cnf).multiply();
		}
		return cnf;
	}

	/**
	 * WhereElement is already in DNF. Call recursively for elements and remove useless
	 * parentheses.
	 * 
	 * @return WhereElement in DNF
	 */
	public WhereElement toDNF() {
		OrWhereElement dnf = new OrWhereElement();
		for (WhereElement where : elements) {
			dnf.add(where.toDNF());
		}
		if (dnf.size() == 0) return dnf;
		return dnf.removeParThisLevel();
	}
	
	/**
	 * Multiplies WhereElement. 
	 * 
	 * Part3: a or (b and c) = (a or b) and (a or c)
	 * 
	 * @return multiplied WhereElement
	 */
	public WhereElement multiply() {
		List<ListOfWhereElements> helpAND = new ArrayList<ListOfWhereElements>();
		ListOfWhereElements helpOR = new ListOfWhereElements();
		for (int i = 0; i < this.size(); i++) {
			if (this.get(i) instanceof AndWhereElement) {
				helpAND.add(this.get(i).elements);
			} else {
				helpOR.add(this.get(i));
			}
		}
		if (helpAND.size() == 0) {
			return new OrWhereElement(helpOR);
		}
		int newsize = 1;
		for (int i = 0; i < helpAND.size(); i++) {
			newsize *= helpAND.get(i).size();
		}
		AndWhereElement result = new AndWhereElement();
		int[] itemArray = new int[helpAND.size()];
		for (int i = 0; i < itemArray.length; i++) {
			// First position
			itemArray[i] = 0;
		}
		for (int i = 0; i < newsize; i++) {
			boolean calculatedNextTuple = false;
			WhereElement help = new OrWhereElement();
			help.addAll(helpOR);
			for (int j = 0; j < helpAND.size(); j++) {
				help.add(helpAND.get(j).get(itemArray[j]));
				// Calculate the next tuple of AND Filters
				if (!calculatedNextTuple) {
					// Increment the value of the itemArray
					if (itemArray[j] >= helpAND.get(j).size() - 1) {
						// All Filters in this list added (cannot increment value)
						itemArray[j] = 0;
					} else {
						itemArray[j] = itemArray[j] + 1;
						calculatedNextTuple = true;
					}
				}
			}
			result.add(help.removeParThisLevel());
		}
		
		return result;
	}
	
	/**
	 * Simplifies this WhereElement, not implemented yet.
	 * 
	 * @return this.clone()
	 */
	public WhereElement simplify() {
		// TODO Auto-generated method stub
		return this.clone();
	}
	
	/**
	 * Removes useless parentheses.
	 * 
	 * a or (b or c) = a or b or c
	 * 
	 * @return WhereElement without useless parentheses
	 */
	public WhereElement removeParThisLevel() {
		OrWhereElement result = new OrWhereElement();
		for (int i=0; i<this.size(); i++) {
			if (this.get(i) instanceof OrWhereElement) {
				result.addAll(this.get(i).elements);
			} else {
				result.add(this.get(i));
			}
		}
		return result;
	}
	
	/**
	 * String representation of this WhereElement.
	 * 
	 * @return string representation
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append('(');
		for (WhereElement where : elements) {
			buffer.append(where.toString());
			buffer.append(" OR ");
		}
		return buffer.substring(0, buffer.length() - 4) + ")";
	}

}
