package sqlindexwizard.filter;

import java.util.ArrayList;

import Zql.ZSelectItem;

/**
 * Representation of a select element.
 * 
 * @author Benjamin Daeumlich
 *
 */

public class SelectElement {

	private String column = "";
	private String table = "";
	private String aggregate= "";
	private boolean isWildcard = false;
	private ArrayList<String> possibleTables = new ArrayList<String>();
	private int type = -1;
	private String[][] aggregateFunctions= {{"","AVG","SUM"},{"MIN","MAX","FIRST","LAST"},{"COUNT"}};
	
	/**
	 * Standard constructor.
	 * 
	 */
	public SelectElement() {
		super();
	}
 	
	/**
	 * Constructor, creates new select element from selectString.
	 * 
	 * @param selectString
	 */
	public SelectElement(String selectString) {
		ZSelectItem select = new ZSelectItem(selectString);
		if (select.isWildcard()) {
			isWildcard = true;
			type = 0;
			return;
		}
		if (select.getAggregate() != null) {
			aggregate = select.getAggregate();
			int beginning = selectString.indexOf("(");
			int ending = selectString.indexOf(")");
			selectString = selectString.substring(beginning+1, ending).trim();
		}
		select = new ZSelectItem(selectString);
		this.column = select.getColumn().toUpperCase();
		if (select.getTable() != null) {
			table = select.getTable().toUpperCase();
		}
		setType(aggregate);
	}
	
	/**
	 * Returns true if the table of this SelectElement is known.
	 * 
	 * @return true if table is set
	 */
	public boolean hasTable() {
		return (!table.equals(""));
	}
	
	/**
	 * Sets the type of the select element according to its aggregate function.
	 * 
	 * @param aggregate
	 */
	private void setType(String aggregate) {
		for (int i = 0; i < aggregateFunctions.length; i++) {
			for (int j = 0; j < aggregateFunctions[i].length; j++) {
				if (aggregate.equals(aggregateFunctions[i][j])) {
					type = i;
					return;
				}
			}
		}
	}
	
	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(Object o) {
		if (o instanceof SelectElement) {
			return equals((SelectElement) o);
		} else {
			return false;
		}
	}

	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(SelectElement it) {
		if (this.column.equals(it.column) && this.table.equals(it.table) && this.type == it.type) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Generating hashCode for SelectElement.
	 * 
	 * @return hashCode
	 */
	public int hashCode(){
		return this.column.hashCode() + this.table.hashCode() + this.aggregate.hashCode() + String.valueOf(this.isWildcard).hashCode();
	}
	
	/**
	 * Clones the select element
	 * 
	 * @return clone
	 */
	public SelectElement clone() {
		SelectElement clone = new SelectElement();
		clone.setColumn(this.column);
		clone.setTable(this.table);
		clone.setAggregate(this.aggregate);
		clone.setWildcard(this.isWildcard);
		clone.type = this.type;
		return clone;
	}
	
	/**
	 * @return the column
	 */
	public String getColumn() {
		return column;
	}

	/**
	 * @param column the column to set
	 */
	public void setColumn(String column) {
		this.column = column;
	}

	/**
	 * @return the table
	 */
	public String getTable() {
		return table;
	}

	/**
	 * @param table the table to set
	 */
	public void setTable(String table) {
		this.table = table;
	}

	/**
	 * @return the aggregate
	 */
	public String getAggregate() {
		return aggregate;
	}

	/**
	 * @param aggregate the aggregate to set
	 */
	public void setAggregate(String aggregate) {
		this.aggregate = aggregate;
	}

	/**
	 * @return the type
	 */
	public int getType() {
		return type;
	}
	
	/**
	 * @return the possibleTables
	 */
	public ArrayList<String> getPossibleTables() {
		return possibleTables;
	}

	/**
	 * @param possibleTables the possibleTables to set
	 */
	public void setPossibleTables(ArrayList<String> possibleTables) {
		this.possibleTables = possibleTables;
	}

	/**
	 * @return the isWildcard
	 */
	public boolean isWildcard() {
		return isWildcard;
	}

	/**
	 * @param isWildcard the isWildcard to set
	 */
	public void setWildcard(boolean isWildcard) {
		this.isWildcard = isWildcard;
	}

	public String toString() {
		String result = "";
		if (isWildcard) {
			return "*";
		}
		if (!aggregate.equals("")) {
			result += aggregate + "(";
		}
		if (!table.equals("")) {
			result += table + ".";
		}
		result += column;
		if (!aggregate.equals("")) {
			result += ")";
		}
		return result;
	}
}
