package sqlindexwizard.indexadvisor;

/**
 * Representation of an index.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class Index {

	private String indexName;
	private String indexTableAndColumn;
	private String indexCreationText;
	private double size = 0.0;
	private double benefit = 0.0;
	private boolean userDefined = false;
	private boolean systemRequired = false;
	private boolean exists = false;
	private long nrLeafInitial = 0;
	
	/**
	 * Constructor sets indexName variable.
	 * 
	 * @param indexName
	 */
	public Index(String indexName) {
		this.indexName = indexName;
	}
	
	/**
	 * @return the indexTableAndColumn
	 */
	public String getIndexTableAndColumn() {
		return indexTableAndColumn;
	}

	/**
	 * @param indexTableAndColumn the indexTableAndColumn to set
	 */
	public void setIndexTableAndColumn(String indexTableAndColumn) {
		this.indexTableAndColumn = indexTableAndColumn;
	}

	/**
	 * Getting indesCreationText.
	 * 
	 * @return the indexCreationText
	 */
	public String getIndexCreationText() {
		return indexCreationText;
	}

	/**
	 * Setting indexCreationText
	 * 
	 * @param indexCreationText the indexCreationText to set
	 */
	public void setIndexCreationText(String indexCreationText) {
		this.indexCreationText = indexCreationText;
	}

	/**
	 * Getting the indexName.
	 * 
	 * @return the indexName
	 */
	public String getIndexName() {
		return indexName;
	}

	/**
	 * Setting the size.
	 * 
	 * @param size new value
	 */
	public void setSize(double size) {
		this.size = size;
	}
	
	/**
	 * Getting the size.
	 * 
	 * @return size
	 */
	public double getSize() {
		return this.size;
	}
	
	/**
	 * Setting the benefit.
	 * 
	 * @param benefit new value
	 */
	public void setBenefit(double benefit) {
		this.benefit = benefit;
	}
	
	/**
	 * Getting the benefit.
	 * 
	 * @return benefit
	 */
	public double getBenefit() {
		return this.benefit;
	}
	
	/**
	 * Getting userDefined
	 * 
	 * @return userDefined
	 */
	public boolean isUserDefined() {
		return userDefined;
	}

	/**
	 * Setting userDefined.
	 * 
	 * @param userDefined value
	 */
	public void setUserDefined(boolean userDefined) {
		this.userDefined = userDefined;
	}

	/**
	 * Getting systemRequired.
	 * 
	 * @return systemRequired
	 */
	public boolean isSystemRequired() {
		return systemRequired;
	}

	/**
	 * Setting systemRequired.
	 * 
	 * @param systemRequired value
	 */
	public void setSystemRequired(boolean systemRequired) {
		this.systemRequired = systemRequired;
	}

	/**
	 * Is it an existing index?
	 * 
	 * @return exists
	 */
	public boolean isExists() {
		return exists;
	}

	/**
	 * Setting exists.
	 * 
	 * @param exists value
	 */
	public void setExists(boolean exists) {
		this.exists = exists;
	}

	/**
	 * Getting the initial nr of leafs.
	 * 
	 * @return nrLeafInitial
	 */
	public long getNrLeafInitial() {
		return nrLeafInitial;
	}

	/**
	 * Setting the initial nr of leafs.
	 * 
	 * @param nrLeafInitial initial nr of leafs to set
	 */
	public void setNrLeafInitial(long nrLeafInitial) {
		this.nrLeafInitial = nrLeafInitial;
	}

	/**
	 * Increases benefit.
	 * 
	 * @param benefit value to increase
	 */
	public void increaseBenefit(double benefit) {
		this.benefit += benefit;
	}
	
	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(Object o) {
		if (o instanceof Index) {
			return equals((Index) o);
		} else {
			return false;
		}
	}

	/**
	 * Tests for equality of two Objects.
	 * 
	 * @return true if they are equal
	 */
	public boolean equals(Index index) {
		return this.indexName.equals(index.indexName);
	}
	
	/**
	 * String representation of an index object.
	 * 
	 * @return String representation
	 */
	public String toString() {
		return "IndexName: " + this.indexName + "(" + this.getIndexTableAndColumn() + ")\tSize: " + this.size + " MB\tBenefit: " + this.benefit + "\tUD: " + this.userDefined + "\tSR: " + this.systemRequired + "\tExists: " + this.exists + "\tNLeaf: " + this.nrLeafInitial + "\n";
	}
	
	/**
	 * String representation of an index object for debugging.
	 * 
	 * @return String representation
	 */
	public String toStringName() {
		return this.indexName + "(" + this.getIndexTableAndColumn() + ")";
	}
	
}
