package sqlindexwizard.indexadvisor;

import java.util.ArrayList;

/**
 * Represents a List of indexes.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class ListOfIndexes extends ArrayList<Index> {

	private static final long serialVersionUID = -5588712179045880787L;
	
	/**
	 * Converts the List into an array.
	 * 
	 * @return indexArray
	 * 
	 */
	public Index[] toArray() {
		return super.toArray(new Index[this.size()]);
	}
	
	/**
	 * Getting the benefit of all Indexes in the list.
	 * 
	 * @return benefit
	 */
	public double getBenefit() {
		double result = 0.0;
		for (int i = 0; i < this.size(); i++) {
			result += this.get(i).getBenefit();
		}
		return result;
	}

	/**
	 * Getting the size of all Indexes in the list.
	 * 
	 * @return size
	 */
	public double getIndexSize() {
		double result = 0.0;
		for (int i = 0; i < this.size(); i++) {
			result += this.get(i).getSize();
		}
		return result;
	}
	
	/**
	 * String representation of ListOfIndexes in short form (indexname(colum)).
	 * 
	 * @return string representation
	 */
	public String toStringName() {
		String result = "[";
		for (int i = 0; i < this.size(); i++) {
			result += this.get(i).toStringName() + ", ";
		}
		if (result.length()>1) {
			result = result.substring(0,result.length()-2);
		}
		return result + "]";
	}

}
