package sqlindexwizard.util;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.log4j.Logger;

/**
 * A class for configurations of properties files.
 *  
 */
public class Config {
	
	/*
	 * +------------------------------------------------+
	 * |                    Constants                   |
	 * +------------------------------------------------+
	 */
	
	/** the file for properties configuration */
	public static final String propertiesFile = "sqlindexwizard.properties";
	
	
	/*
	 * +------------------------------------------------+
	 * |                    Variables                   |
	 * +------------------------------------------------+
	 */
	
	/* ================ Private Variables =============== */
	
	/**	the configuration */
	private static CompositeConfiguration config = null;
	/** a logger for the Class */
	private static Logger logger = Log4jConfiguration.getLogger();
	
	static {
		config = new CompositeConfiguration();
		try {
			config.addConfiguration(new PropertiesConfiguration(propertiesFile));
		} catch (ConfigurationException e) {
			logger.error("Error loading properties. Aborting.", e);
			throw new RuntimeException(e);
		}
	}
	
	
	/*
	 * +------------------------------------------------+
	 * |                     Methods                    |
	 * +------------------------------------------------+
	 */
	
   	/* ================== Get Property ================== */
	
	/**
	 * Gets den value of the property with the given name.
	 * @param propertyName name of the property
	 * @return the value of the property as a String;
	 * <code>null</code>, if the property does not exist
	 */
	public static String getPropertyString(String propertyName) {
		return config.getString(propertyName);
	}
	
	/**
	 * Gets den value of the property with the given name or the given default value.
	 * @param propertyName name of the property
	 * @param defValue default value, if the property does not exist
	 * @return the value of the property or the default value as a String
	 */
	public static String getPropertyString(String propertyName, String defValue) {
		return config.getString(propertyName, defValue);
	}
	
	/**
	 * Gets the integer value of the property with the given name.
	 * @param propertyName name of the property
	 * @return the value of the property as an int;
	 * <code>null</code>, if the property does not exist
	 */
	public static int getPropertyInt(String propertyName) {
		return config.getInt(propertyName);
	}
	
	/**
	 * Gets the long value of the property with the given name.
	 * @param propertyName name of the property
	 * @return the value of the property as an long;
	 * <code>null</code>, if the property does not exist
	 */
	public static long getPropertyLong(String propertyName) {
		return config.getLong(propertyName);
	}
	
	/**
	 * Gets the double value of the property with the given name.
	 * @param propertyName name of the property
	 * @return the value of the property as a double;
	 * <code>null</code>, if the property does not exist
	 */
	public static double getPropertyDouble(String propertyName) {
		return config.getDouble(propertyName);
	}
	
	/**
	 * Sets the property to a given string.
	 * 
	 * @param propertyName name of the property
	 * @param value new value
	 */
	public static void setPropertyString(String propertyName, String value) {
		config.getConfiguration(0).setProperty(propertyName, value);
		try {
			((PropertiesConfiguration)config.getConfiguration(0)).save();
		} catch (ConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
   	/* ================ Getter and Setter =============== */

	/**
	 * Getting the config.
	 * 
	 * @return config
	 */
	public static CompositeConfiguration getConfig() {
		return config;
	}
}
