package sqlindexwizard.util;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.xml.DOMConfigurator;

/**
 * A class for configuration of Log4J. 
 * 
 */
public class Log4jConfiguration {
	
	/*
	 * +------------------------------------------------+
	 * |                    Constants                   |
	 * +------------------------------------------------+
	 */

	/** specifies which log file to use (prperties file, XML file) */
	private static enum FileType {
		/** the configuration file is a properties file */
		PROPERTIES_FILE,
		/** the configuration file is a XML file */
		XML_FILE,
		/** there is no configuration file, use the configuration in the java code */
		CODE,
		/** use the default configuration */
		DEFAULT
	};

	/** the property file for log4j logging */
	public static final String LOGGINGFILEPROP = "log4j.properties";
	/** the XML file for log4j logging */
	public static final String LOGGINGFILEXML = "log4j.xml";
	
	
	/*
	 * +------------------------------------------------+
	 * |                    Variables                   |
	 * +------------------------------------------------+
	 */
	
	/* ================ Private Variables =============== */
	
	/** which log4j file should be used */
	private static FileType type = FileType.PROPERTIES_FILE;
	
	
	/*
	 * +------------------------------------------------+
	 * |                     Methods                    |
	 * +------------------------------------------------+
	 */
	
	/* ================= Configuration ================== */
	
	/**
	 * Configure the input (filename, loglevel, appender, layout).
	 */
	public static void configureLogging() {
		// Create logger from properties or XML file
		switch (type) {
			case PROPERTIES_FILE:
				PropertyConfigurator.configureAndWatch(LOGGINGFILEPROP);
				break;
			case XML_FILE:
				DOMConfigurator.configureAndWatch(LOGGINGFILEXML);
				break;
			case CODE:
				// Set up a simple configuration that logs on the console.
				ConsoleAppender stdout = new ConsoleAppender(new PatternLayout("%d{ISO8601} %-5p [%t] %c: %m%n"));
				Logger rootLogger = Logger.getRootLogger();
				rootLogger.addAppender(stdout);
				rootLogger.setLevel(Level.INFO);
				break;
			default:
				// use a default logger
				BasicConfigurator.configure();
				break;
		}
	}

	
	/* =================== Get Logger =================== */
	
	/**
	 * Gets a logger for a calling class.
	 * @return a logger
	 */
	public static Logger getLogger() {
		// get the calling methode (StackTrace[0] = this methode)
		StackTraceElement directCaller = new Throwable().getStackTrace()[1];
		return Logger.getLogger(directCaller.getClassName());
	}
}
