package sqlindexwizard.util;

import java.io.IOException;

import jline.ConsoleReader;
import jline.Terminal;

/**
 * A simple progress-bar. Does not work in Eclipse console
 * due to a bug in eclipse.
 * 
 * @author Benjamin Daeumlich
 * 
 */
public class ProgressBar {
	
	private Terminal terminal ;
	private ConsoleReader reader;

	/**
	 * Constructor initializes terminal and reader.
	 * 
	 */
	public ProgressBar() {
		try {
		        terminal = Terminal.setupTerminal();
		        reader = new ConsoleReader();
		        terminal.beforeReadLine(reader, "", (char)0);
		} catch (Exception e) {
		        e.printStackTrace();
		        terminal = null;
		}
	}
	
	/**
	 * Setting progress (completed of total).
	 * 
	 * @param completed
	 * @param total
	 */
	public void setProgress(int completed, int total) {
		if (terminal == null)
			return;
		int w = reader.getTermwidth();
		int progress = 0;
		if (total == 0) {
			progress = 20;
		} else {
			progress = (completed * 20) / total;
		}
        String totalStr = String.valueOf(total);
        String percent = String.format("%0"+totalStr.length()+"d/%s [", completed, totalStr);
        String result = percent + repetition("=", progress)+ repetition(" ", 20 - progress) + "]";
        try {
        	reader.getCursorBuffer().clearBuffer();
            reader.getCursorBuffer().write(result);
            reader.setCursorPosition(w);
            reader.redrawLine();
        } catch (IOException e) {
        	e.printStackTrace();
        }
	}
	
	/**
	 * Repeats value count times and return the result as a string.
	 * 
	 * @param value
	 * @param count
	 * @return value count times
	 */
	public String repetition(String value, int count) {
		String result = "";
		for (int i = 0; i < count; i++) {
			result += value;
		}
		return result;
	}
	                                                           
}