package sqlindexwizard.util;

/**
 * TestClass for similarity calculation.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class SimTest {

	/**
	 * Main-Method.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		double[] q1 = {0.74};
		double[] q2 = {0.14};
		double[] q3 = {0.1};
		System.out.println("sim(Q1,Q2) = " + sim(q1,q2));
		System.out.println("sim(Q2,Q3) = " + sim(q2,q3));
		System.out.println("sim(Q1,Q3) = " + sim(q1,q3));
		System.out.println("\nCorrect: sim(Q1,Q2) >= sim(Q2,Q3) >= sim(Q1,Q3)");
	}
	
	/**
	 * Computes absolute value of vector q.
	 * 
	 * @param q
	 * @return abs of q
	 */
	public static double abs(double[] q) {
		double result = 0.0;
		for (int i = 0; i < q.length; i++) {
			result += q[i]*q[i];
		}
		return Math.sqrt(result);
	}
	
	/**
	 * Computes scalar product of vectors q1 and q2.
	 * 
	 * @param q1
	 * @param q2
	 * @return scalar product
	 */
	public static double prod(double[] q1, double[] q2) {
		double result = 0;
		if (q1.length != q2.length) { 
			System.out.println("Different dimensions!");
			System.exit(-1);
		} else {
			for (int i = 0; i < q1.length; i++) {
				result += q1[i]*q2[i];
			}
		}
		return result;
	}
	
	/**
	 * Computes similarity of q1 and q2.
	 * 
	 * @param q1
	 * @param q2
	 * @return similarity
	 */
	public static double sim(double[] q1, double[] q2) {
		return (prod(q1,q2)/(abs(q1)*abs(q2)));
	}

}
