package sqlindexwizard.workload;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import Zql.*;

import sqlindexwizard.application.Input;
import sqlindexwizard.filter.EmptyWhereElement;
import sqlindexwizard.filter.ItemWhereElement;
import sqlindexwizard.filter.WhereElement;

/**
 * Represents an DeleteQuery.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class DeleteQuery extends Query {

	private String table = "";
	private WhereElement where = null;
	
	private static Logger logger = Logger.getLogger(DeleteQuery.class);
	
	/**
	 * Constructor generates new Delete Query from statement, extracts table and where part.
	 * 
	 * @param st statement
	 */
	public DeleteQuery(ZStatement st) {
		super();
		table = ((ZDelete)st).getTable().toUpperCase();
		extractWhere((ZDelete)st);
		where = where.removePar();
		if (Input.toCNF) {
			where = where.toCNF();
		}
		if (Input.toDNF) {
			where = where.toDNF();
		}
		if (Input.simplify) {
			where = where.simplify();
		}
		logger.debug("Query ItemList: " + this.getItemList());
		logger.info("Generated new DeleteQuery: " + this.toString());
	}
	
	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public DeleteQuery() {
		super();
	}
	
	/**
	 * Extracts where part of the delete query.
	 * 
	 * @param select query
	 */
	public void extractWhere(ZDelete select) {
		ZExp whereclause = select.getWhere();
		if (whereclause instanceof ZQuery) {
			logger.error("No Subquerys supported!");
		} else if (whereclause instanceof ZExpression) {
			where = WhereElement.parseWhereClause(whereclause);	
		} else {
			where = new EmptyWhereElement();
		}
	}
	
	/**
	 * Converts where part into DNF.
	 * 
	 */
	public void whereClauseToDNF(){
		where = where.toDNF();
	}
	
	/**
	 * Converts where part into CNF.
	 */
	public void whereClauseToCNF(){
		where = where.toCNF();
	}
	
	/**
	 * Simplyfies where part.
	 * 
	 */
	public void whereClauseSimplify(){
		where = where.simplify();
	}
	
	@Override
	public List<ItemWhereElement> getItemList() {
		List<ItemWhereElement> temp = where.getItemList();
		List<ItemWhereElement> result = new ArrayList<ItemWhereElement>();
		for(int i = 0; i < temp.size(); i++) {
			if (temp.get(i).hasTable()) {
				result.add(temp.get(i));
				Workload.getItemList().add(temp.get(i));
			} else {
				temp.get(i).setTable(table);
				result.add(temp.get(i));
				Workload.getItemList().add(temp.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Getting Tables of the delete query.
	 * 
	 * @return tables
	 */
	@Override
	public ArrayList<String> getTables() {
		ArrayList<String> tables = new ArrayList<String>();
		tables.add(table);
		return tables;
	}
	
	/**
	 * Getting the where-clause.
	 * 
	 * @return where-clause
	 */
	public WhereElement getWhere() {
		return where;
	}
	
	/**
	 * Clones this DeleteQuery.
	 * 
	 * @return clone
	 */
	public DeleteQuery clone() {
		DeleteQuery clone = new DeleteQuery();
		clone.table = this.table;
		clone.where = this.where.clone();
		return clone;
	}
	
	/**
	 * String representation of delete query.
	 * 
	 * @return string representation
	 */
	public String toString() {
		String result = "DELETE FROM " + table;
		if (!(where instanceof EmptyWhereElement )) {
			result += " WHERE " + where.toString();
		}
		result += ";";
		return result;
	}

}
