package sqlindexwizard.workload;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;

import Zql.*;

import sqlindexwizard.filter.ItemWhereElement;

/**
 * Represents an InsertQuery.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class InsertQuery extends Query {
	
	private String table = "";
	private ArrayList<String> columns = new ArrayList<String>();
	private ArrayList<String> values = new ArrayList<String>();
	
	private static Logger logger = Logger.getLogger(InsertQuery.class);
	
	/**
	 * Constructor generates Insert query from statement, extracts table, columns and values.
	 * 
	 * @param st Statement
	 */
	public InsertQuery(ZStatement st) {
		super();
		table = ((ZInsert)st).getTable().toUpperCase();
		extractColumns((ZInsert)st);
		extractValues((ZInsert)st);
		logger.info("Generated new InsertQuery: " + this.toString());
	}
	
	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public InsertQuery() {
		super();
	}
	
	/**
	 * Extracts columns from insert query.
	 * 
	 * @param insert query
	 */
	@SuppressWarnings("unchecked")
	public void extractColumns(ZInsert insert) {
		Vector<String> cols = (Vector<String>)insert.getColumns();
		if (cols != null) {
			for (int i = 0; i < cols.size(); i++) {
				columns.add(cols.get(i).toUpperCase());
			}
		}
	}
	
	/**
	 * Extracts values from insert query.
	 * 
	 * @param insert query
	 */
	@SuppressWarnings("unchecked")
	public void extractValues(ZInsert insert) {
		Vector<ZExp> vals = (Vector<ZExp>)insert.getValues();
		if (vals == null) {
			logger.error("Subqueries not supported!");
		} else {
			for (int i = 0; i < vals.size(); i++) {
				values.add(((ZConstant)vals.get(i)).getValue());
			}
		}
	}
	
	/**
	 * Getting itemList. Is null because here is no where part.
	 * 
	 * @return null
	 */
	@Override
	public List<ItemWhereElement> getItemList() {
		return null;
	}
	
	/**
	 * Getting Tables of the insert query.
	 * 
	 * @return tables
	 */
	@Override
	public ArrayList<String> getTables() {
		ArrayList<String> tables = new ArrayList<String>();
		tables.add(table);
		return tables;
	}
	
	/**
	 * Getting columns.
	 * 
	 * @return the columns
	 */
	public ArrayList<String> getColumns() {
		return columns;
	}

	/**
	 * Clones this InsertQuery.
	 * 
	 * @return clone
	 */
	public InsertQuery clone() {
		InsertQuery clone = new InsertQuery();
		clone.table = this.table;
		clone.columns = this.columns;
		clone.values = this.values;
		return clone;
	}
	
	/**
	 * String representation of insert query.
	 * 
	 * @return string representation
	 */
	public String toString() {
		String result = "INSERT INTO " + table + " ";
		if (columns.size() > 0) {
			result += "(";
			for (int i = 0; i < columns.size(); i++) {
				result += columns.get(i) + ", ";
			}
			result = result.substring(0, result.length()-2);
			result += ") ";
		}
		result += "VALUES (";
		for (int i = 0; i < values.size(); i++) {
			result += values.get(i) + ", ";
		}
		result = result.substring(0, result.length()-2);
		result += ");";
		return result;
	}

}
