package sqlindexwizard.workload;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import sqlindexwizard.filter.ItemWhereElement;

import Zql.*;

/**
 * Representation of a Query.
 * 
 * @author Benjamin Daeumlich
 *
 */
public abstract class Query {
	
	private static Logger logger = Logger.getLogger(Query.class);
	
	private int count = 1;
	
	/**
	 * Generates Query from Statement. 
	 * 
	 * @param st Statement
	 * @return QueryInstance
	 */
	public static Query generateQuery(ZStatement st) {
		logger.debug("Generating new Query!");
		if (st instanceof ZQuery) {
			if (((ZQuery)st).getSet() != null) {
				return new InvalidQuery(st.toString().toUpperCase() + ";");
			} else {
				SelectQuery select = new SelectQuery((ZQuery)st);
				if (!select.isValid()) {
					return new InvalidQuery(st.toString().toUpperCase() + ";");
				} else {
					return select;
				}
			}
		} else if (st instanceof ZUpdate) {
			return new UpdateQuery((ZUpdate)st);
		} else if (st instanceof ZDelete) {
			return new DeleteQuery((ZDelete)st);
		} else if (st instanceof ZInsert) {
			if (((ZInsert)st).getQuery() != null) {
				return new InvalidQuery(st.toString().toUpperCase() + ";");
			} else {
				return new InsertQuery((ZInsert)st);
			}
		} else if (st instanceof ZTransactStmt){
			return new OtherQuery(st);
		} else if (st instanceof ZLockTable){
			return new OtherQuery(st);
		} else {
			return new OtherQuery(st);
		}
	}
	
	/**
	 * Converts where clause to DNF if there is a where clause.
	 * 
	 */
	public void whereClauseToDNF(){
	}
	
	/**
	 * Converts where clause to CNF if there is a where clause.
	 * 
	 */
	public void whereClauseToCNF(){
	}
	
	/**
	 * Simplyfies where clause if there is a where clause.
	 * 
	 */
	public void whereClauseSimplify(){
	}
	
	/**
	 * Clones this query.
	 * 
	 */
	public abstract Query clone();
	
	/**
	 * Getting ITemList from the where part of the query. If there is no where part
	 * null will be retured.
	 * 
	 * @return itemList
	 */
	public abstract List<ItemWhereElement> getItemList();
	
	/**
	 * Getting the list of referenced tables.
	 * 
	 * @return list of referenced tables
	 */
	public abstract ArrayList<String> getTables();
	
	/**
	 * Getting query count.
	 * 
	 * @return count
	 */
	public int getCount() {
		return count;
	}

	/**
	 * Setting query count.
	 * 
	 * @param count new count
	 */
	public void setCount(int count) {
		this.count = count;
	}
	
	/**
	 * Converts this query into a String.
	 * 
	 * @return a String representation of the query.
	 */
	public abstract String toString();
}
