package sqlindexwizard.workload;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import org.apache.log4j.Logger;

import Zql.*;

import sqlindexwizard.application.Input;
import sqlindexwizard.filter.EmptyWhereElement;
import sqlindexwizard.filter.ItemWhereElement;
import sqlindexwizard.filter.WhereElement;

/**
 * Represents an UpdateQuery.
 * 
 * @author Benjamin Daeumlich
 *
 */
public class UpdateQuery extends Query {

	private String table = "";
	private WhereElement where = null;
	private Hashtable<String, String> setvalues = new Hashtable<String, String>();
	private ArrayList<String> columns = new ArrayList<String>();
	
	private static Logger logger = Logger.getLogger(UpdateQuery.class);
	
	/**
	 * Constructor, generates new UpdateQuery from Statement.
	 * 
	 * @param st Statement
	 */
	public UpdateQuery(ZStatement st) {
		super();
		table = ((ZUpdate)st).getTable().toUpperCase();
		extractSet((ZUpdate)st);
		extractWhere((ZUpdate)st);
		where = where.removePar();
		if (Input.toCNF) {
			where = where.toCNF();
		}
		if (Input.toDNF) {
			where = where.toDNF();
		}
		if (Input.simplify) {
			where = where.simplify();
		}
		columns.addAll(setvalues.keySet());
		logger.debug("Query ItemList: " + this.getItemList());
		logger.info("Generated new UpdateQuery: " + this.toString());
	}
	
	/**
	 * Constructor calls super constructor.
	 * 
	 */
	public UpdateQuery() {
		super();
	}
	
	/**
	 * Extracts set-part of Update statement.
	 * 
	 * @param set setpart
	 */
	@SuppressWarnings("unchecked")
	public void extractSet(ZUpdate set) {
		Hashtable<String, ZExp> temp = (Hashtable<String, ZExp>)set.getSet();
		Enumeration<String> temp2 = temp.keys();
		while (temp2.hasMoreElements()) {
			String element = temp2.nextElement();
			ZExp value = temp.get(element);
			setvalues.put(element.toUpperCase(), ((ZConstant)value).getValue());
		}
	}
	
	/**
	 * Extracts Where-part of update statement.
	 * 
	 * @param select where part
	 */
	public void extractWhere(ZUpdate select) {
		ZExp whereclause = select.getWhere();
		if (whereclause instanceof ZQuery) {
			logger.error("No Subquerys supported!");
		} else if (whereclause instanceof ZExpression) {
			where = WhereElement.parseWhereClause(whereclause);
		} else {
			where = new EmptyWhereElement();
		}
	}
	
	/**
	 * Converts where part into DNF.
	 * 
	 */
	public void whereClauseToDNF(){
		where = where.toDNF();
	}
	
	/**
	 * Converts where part into CNF.
	 */
	public void whereClauseToCNF(){
		where = where.toCNF();
	}
	
	/**
	 * Simplyfies where part.
	 * 
	 */
	public void whereClauseSimplify(){
		where = where.simplify();
	}

	/**
	 * returns ItemList form where-part
	 * 
	 * @return itemList in where-part.
	 */
	@Override
	public List<ItemWhereElement> getItemList() {
		List<ItemWhereElement> temp = where.getItemList();
		List<ItemWhereElement> result = new ArrayList<ItemWhereElement>();
		for(int i = 0; i < temp.size(); i++) {
			if (temp.get(i).hasTable()) {
				result.add(temp.get(i));
				Workload.getItemList().add(temp.get(i));
			} else {
				temp.get(i).setTable(table);
				result.add(temp.get(i));
				Workload.getItemList().add(temp.get(i));
			}
		}
		return result;
	}
	
	/**
	 * Getting Tables of the update query.
	 * 
	 * @return tables
	 */
	@Override
	public ArrayList<String> getTables() {
		ArrayList<String> tables = new ArrayList<String>();
		tables.add(table);
		return tables;
	}
	
	/**
	 * Getting columns.
	 * 
	 * @return the columns
	 */
	public ArrayList<String> getColumns() {
		return columns;
	}

	/**
	 * Getting the where-clause.
	 * 
	 * @return where-clause
	 */
	public WhereElement getWhere() {
		return where;
	}
	
	/**
	 * Clones this UpdateQuery.
	 * 
	 * @return clone
	 */
	public UpdateQuery clone() {
		UpdateQuery clone = new UpdateQuery();
		clone.table = this.table;
		clone.where = this.where.clone();
		clone.setvalues = this.setvalues;
		return clone;
	}
	
	/**
	 * String-Representation of Update query.
	 * 
	 * @return String-Representation
	 */
	public String toString() {
		String result = "UPDATE " + table + " SET ";
		String help = setvalues.toString();
		help = help.substring(1);
		help = help.substring(0, help.length()-1);
		result += help;		
		if (!(where instanceof EmptyWhereElement )) {
			result += " WHERE " + where.toString();
		}
		result += ";";
		return result;	
	}

}
